#include <stdio.h>
#include <string.h>

#include "chip8.h"

#define SDL2

#ifdef SDL2
#include "sdl2_ui.h"

struct chip8_callbacks cb = {
	.tick      = sdl2_tick,
	.draw      = sdl2_draw,
	.sound     = sdl2_sound,
	.getkey    = sdl2_getkey,
	.waitkey   = sdl2_waitkey,
};
#else
static int dummytick(void)
{ printf("TICK\n"); return 0; }

static void dummydraw(int x, int y, int color)
{ printf("DRAW (%02d,%02d) -> %02d\n", x, y, color); }

static void dummysound(int enable)
{ printf("SOUND %s\n", enable ? "on" : "off"); }

static int dummygetkey(int key)
{ printf("GETKEY %d\n", key); return 0; }

static int dummywaitkey(void)
{ printf("WAITKEY\n"); while(1) usleep(10); return -1; }

struct chip8_callbacks cb = {
	.tick    = dummytick,
	.draw    = dummydraw,
	.sound   = dummysound,
	.getkey  = dummygetkey,
	.waitkey = dummywaitkey,
};
#endif

int main(int argc, char* argv[])
{
	int nosound = 0;

	if (argc < 2) {
		fprintf(stderr, "Usage: %s program [nobeep]\n", argv[0]);
		return 1;
	}

	if (argc == 3 && !strncmp(argv[2], "no", 2)) {
		printf("Audio disabled.\n");
		nosound = 1;
		cb.sound = NULL;
	}


#ifdef SDL2
	if (sdl2_init("CHIP-8", nosound)) {
		fprintf(stderr, "Failed to initialize SDL2.\n");
		return 1;
	}
#endif

	void* chip8 = chip8_create(&cb);
	if (!chip8) {
		fprintf(stderr, "Failed to initialize CHIP-8.\n");
		return 1;
	}

	if (chip8_load(chip8, argv[1])) {
		fprintf(stderr, "Failed to load CHIP-8 program.\n");
		return 1;
	}

	printf("Program start.\n");
	int ret = chip8_run(chip8);
	printf("Program finished with code %d.\n", ret);

	chip8_destroy(chip8);

#ifdef SDL2
	sdl2_destroy();
#endif

	return 0;
}
